//fm
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

/*==========================================================
Mandelbrot set class.
Displays the mandelbrot fractal in the frame

Must override:
run()			:actual work done here
tuneSettings()	:to set parameters
resetDefaults()	:to set params to default values

===========================================================*/
class Mandelbrot extends Fractal
{
    
    private boolean firstTime=true;
    private double amin = -2.0;
    private double amax =  1.0;
    private double bmin = -1.5;
    private double bmax =  1.5;
    private double alen, blen;
    private double a=0.0;
    private double b=0.0;
    private double x=0.0;
    private double y=0.0;
    private double acenter,bcenter;
    private int zoomin=10;
    private int scaleda,scaledb;
  
    private Graphics offGraphics, onGraphics;
    Image offImage=null;
    private int delay=0;
    private int Index=0;
    int[] pixels;
    private MemoryImageSource source;
    int alpha = 0xff;
    int red = 0xff;
    int green = 0xff;
    int blue = 0xff;
    int times = 255;

  
  	Mandelbrot(FractalMagic f)
  	{
		super(f);

	}
	public void initialize()
    {	
    	setFractalName("Mandelbrot Set of fractals");
    	frameWidth = size().width;
   	 	frameHeight = size().height;
    	alen=amax-amin;
    	blen=bmax-bmin;

    	pixels = new int[frameWidth*frameHeight];

    	onGraphics=getGraphics();

		offImage=this.createImage(frameWidth,frameHeight);
		offGraphics=offImage.getGraphics();


		onGraphics.setFont(new Font("TimesRoman",Font.BOLD,30));
		onGraphics.setColor(Color.red);

	}
	public void run ()
	{	
		Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
		
		System.out.println("Inside run");//for debugging
		try {Thread.currentThread().sleep(3000);}
			catch (InterruptedException e) { }
		System.out.println("wokeup");		

		
		while (t!=null)
		{
			if(Index==1)
			{	int incr=0;
				printStatus("Preparing the canvas.....");
				for(int iy=0;iy<frameHeight;iy++)
					for(int ix=0;ix<frameWidth;ix++)
						pixels[incr++]=alpha<<24 | 0x5f<<16 | 0x5f<< 8 | 0x5f;


				if(firstTime)
				{ 	source=new MemoryImageSource
					 (frameWidth,frameHeight,pixels,0,frameWidth);
					firstTime=false;
				}
				else
				   source.newPixels(0,0,frameWidth,frameHeight);

				offImage=createImage(source);
				paint(onGraphics);
			}// end of index==1

			if(Index==2)
			{
				double x1,y1;
				for(a=amin;a<amax;a+=((amax-amin)/frameWidth))
				{	for(b=bmin;b<bmax;b+=((bmax-bmin)/frameHeight))
					{	
						
						x=0.0;
						y=0.0;
						int iteration=0;
						while((x*x+y*y<=4.0) && (iteration!=times))
						{	x1 = xmapMandelbrot(x,y,a);
							y1 = ymapMandelbrot(x,y,b);
							x = x1;
							y = y1;
							iteration ++;
						}
						if(iteration<=times && iteration>0)
						{	scaleda=scaleX(a,amin,amax);
							scaledb=scaleY(b,bmin,bmax);
							pixels[scaledb*frameWidth+scaleda]=
							alpha<<24 | red<<16 | iteration<<8 | blue;
							
						}
						//else
							//printStatus("Deciding for point [ "+scaleda+" , "+scaledb+" ]...outside");
					}
				}

				source.newPixels(0,0,frameWidth,frameHeight);
				offImage=createImage(source);
			}// end of index==2

			Index++;
			paint(onGraphics);

			try {Thread.currentThread().sleep(delay);}
			catch (InterruptedException e) { }
		}	//end of while(t!=null)
	}	//end of run fn.

	void printStatus(String s)
	{
		this.getGraphics().drawString(s,20,frameHeight-50);
	}
	public void paint(Graphics g)
	{	if(offImage!=null)
		{	g.drawImage(offImage,0,0,frameWidth,frameHeight,null);
			if(Index<=2){
				 g.drawString("Processing......",frameWidth/2-70,frameHeight/2);
			}
		}
	}

	private double xmapMandelbrot(double x, double y, double a)
	{	return x*x-y*y+a;	}
	private double ymapMandelbrot(double x, double y, double b)
	{	return 2.0*x*y+b;	}

	private int scaleX(double x, double xmin, double xmax)
	{	int ivalue;
		if( x >= xmin && x < xmax )
			ivalue = (int) ((x - xmin)*frameWidth/(xmax - xmin));
		else
		{	ivalue=0;
			System.out.println("arg "+x+" out of range:["+xmin+","+xmax+"]");
			System.exit(1);
		}
		return ivalue;
	}

	private int scaleY(double y, double ymin, double ymax)
	{	int jvalue;
		if( y >= ymin && y < ymax )
			jvalue = (int) ((y - ymin)*frameHeight/(ymax - ymin));
		else
		{   jvalue=0;
			System.out.println("arg "+y+" out of range:["+ymin+","+ymax+"]");
			System.exit(2);
		}
		return jvalue;
	}

	public boolean mouseDown (Event e, int x, int y)
	{
		if(x>frameWidth) {x=frameWidth;}
		if(x<0) {x=0;}
		if(y>frameHeight) {y=frameHeight;}
		if(y<0) {y=0;}

		acenter=((double) (x)/(double) (frameWidth))*alen+amin;
		bcenter=((double) (y)/(double) (frameHeight))*blen+bmin;
		amin=acenter-alen/zoomin;
		amax=acenter+alen/zoomin;
		bmin=bcenter-blen/zoomin;
		bmax=bcenter+blen/zoomin;
		alen=amax-amin;
		blen=bmax-bmin;
		if(Math.max(alen,blen)<0.00000003)
		{  	amax=1.0;amin=-2.0;bmin=-1.5;bmax=1.5;
			alen=amax-amin;blen=bmax-bmin;
		}
		Index=1;
		return true;
	}
	
			
	void tuneSettings()
	{
		Frame fr=new Frame("MandelBrot Settings");
		fr.setSize(300,300);
		fr.setVisible(true);
		
		Label lamin=new Label("A Min:  ",Label.RIGHT);
		
		
		add(lamin);
		
	}
	void saveToFile(){	save(offImage);	}
	void quit(){	t=null;this.dispose();main.setVisible(true);}	
	void resetDefaults()
	{
		
		amax=1.0;amin=-2.0;bmin=-1.5;bmax=1.5;
		alen=amax-amin;blen=bmax-bmin;
		firstTime=true;
		Index=1;
	}
	
}
